@@ -133,7 +133,7 @@ gpa_utils_dump_tree_with_level (GPANode *node, gint level, gint follow_reference
 			g_print ("node");
 			break;
 		case GPA_OPTION_TYPE_ITEM:
-			g_print ("item");
+			g_print ("item [%s]", option->value);
 			break;
 		case GPA_OPTION_TYPE_ROOT:
 			g_print ("root");
