@@ -27,6 +27,7 @@
 #include <config.h>
 #include <glib.h>
 #include <gmodule.h>
+#include <locale.h>
 
 #include <libgnomeprint/gnome-print-module.h>
 #include <libgnomeprint/gpa/gpa-model.h>
@@ -37,6 +38,8 @@
 #include <cups/cups.h>
 #include <cups/ppd.h>
 
+#include "libgnomeprint/gnome-print-i18n.h"
+
 /* Argument order: id, name */
 
 xmlChar *model_xml_template = 
@@ -67,7 +70,11 @@ xmlChar *model_xml_template =
 "      </Option>"
 "      <Option Id=\"Job\">"
 "        <Option Id=\"NumCopies\" Type=\"String\" Default=\"1\"/>"
+"        <Option Id=\"NonCollatedCopiesHW\" Type=\"String\" Default=\"true\"/>"
+"        <Option Id=\"CollatedCopiesHW\" Type=\"String\" Default=\"false\"/>"
 "        <Option Id=\"Collate\" Type=\"String\" Default=\"false\"/>"
+"        <Option Id=\"Duplex\" Type=\"String\" Default=\"true\"/>"
+"        <Option Id=\"Tumble\" Type=\"String\" Default=\"false\"/>"
 "        <Option Id=\"PrintToFile\" Type=\"String\" Default=\"false\" Locked=\"true\"/>"
 "        <Option Id=\"FileName\" Type=\"String\" Default=\"output.ps\"/>"
 "      </Option>"
@@ -80,6 +87,8 @@ xmlChar *model_xml_template =
 "  </Options>"
 "</Model>";
 
+G_MODULE_EXPORT void gpa_module_load_data (GPAPrinter *printer);
+
 static GPANode *
 gpa_model_new_from_xml (char *string)
 {
@@ -106,7 +115,9 @@ get_paper_text (ppd_file_t *ppd, ppd_size_t *size)
 {
 	/* This is dumb and slow and ugly and crappy and I hate myself. (Dave)*/
 	int i;
-	
+	char *result;
+	static gboolean warn = FALSE;
+
 	for (i = 0; i < ppd->num_groups; i++) {
 		ppd_group_t *group = &ppd->groups[i];
 		int j;
@@ -117,13 +128,48 @@ get_paper_text (ppd_file_t *ppd, ppd_size_t *size)
 				for (k = 0; k < option->num_choices; k++) {
 					if (!strcmp (option->choices[k].choice,
 						     size->name)) {
-						return g_strdup (option->choices[k].text);
+						result = g_convert (option->choices[k].text, 
+								    strlen (option->choices[k].text), 
+								    "UTF8", 
+								    ppd->lang_encoding, 
+								    NULL, NULL, NULL);
+						if (result != NULL)
+							return result;
+						if (!warn) {
+							warn = TRUE;
+							g_warning ("iconv does not support ppd "
+								   "character encoding: %s, trying "
+								   "CSISOLatin1", 
+								   ppd->lang_encoding);
+						}
+						return g_convert (option->choices[k].text, 
+									  strlen (option->choices[k].text),
+									  "UTF8", 
+									  "CSISOLatin1", 
+									  NULL, NULL, NULL);
 					}
 				}
 			}
 		}
 	}
-	return g_strdup (size->name);
+	
+	result = g_convert (size->name, 
+			    strlen (size->name), 
+			    "UTF8", 
+			    ppd->lang_encoding, 
+			    NULL, NULL, NULL);
+	if (result != NULL)
+		return result;
+	if (!warn) {
+		warn = TRUE;
+		g_warning ("iconv does not support ppd character encoding:"
+			   " %s, trying CSISOLatin1", ppd->lang_encoding);
+	}
+	return g_convert (size->name, 
+			  strlen (size->name), 
+			  "UTF8", 
+			  "CSISOLatin1", 
+			  NULL, NULL, NULL);
 }
 
 static GPANode *
@@ -176,23 +222,30 @@ load_paper_sizes (ppd_file_t *ppd, GPANode *parent)
 		return NULL;
 			
 	for (i = 0; i < ppd->num_sizes; i++) {
-		GPANode *size;
-		gchar *height;
-		gchar *width;
 		gchar *paper_name;
 
 		paper_name = get_paper_text (ppd, &ppd->sizes[i]);
-		size = gpa_option_item_new (node, ppd->sizes[i].name, paper_name);
-		g_free (paper_name);
 
-		width  = g_strdup_printf ("%d", (int)ppd->sizes[i].width);
-		height = g_strdup_printf ("%d", (int)ppd->sizes[i].length);
+		if (paper_name != NULL) {
+			GPANode *size;
+			gchar *height;
+			gchar *width;
+			size = gpa_option_item_new (node, 
+						    ppd->sizes[i].name, 
+						    paper_name);
+			g_free (paper_name);
+			
+			width  = g_strdup_printf ("%d", 
+						  (int)ppd->sizes[i].width);
+			height = g_strdup_printf ("%d", 
+						  (int)ppd->sizes[i].length);
 
-		gpa_option_key_new (size, "Width",  width);
-		gpa_option_key_new (size, "Height", height);
-
-		g_free (width);
-		g_free (height);
+			gpa_option_key_new (size, "Width",  width);
+			gpa_option_key_new (size, "Height", height);
+			
+			g_free (width);
+			g_free (height);
+		}
 	}
 
 	gpa_node_reverse_children (node);
@@ -219,7 +272,34 @@ load_paper_sources (ppd_file_t *ppd, GPANode *parent)
 	for (i = 0; i < option->num_choices; i++)
 		gpa_option_item_new (node,
 				     option->choices[i].choice,
-				     option->choices[i].choice);
+				     option->choices[i].text);
+}
+
+/* load_cups_hold_types */
+/* we could either make an ipp request for job-hold-until-supported */
+/* or list here the cups supported values */
+/* Sicne these values are in fact not printer specific but cups specific, */
+/* We'll just list them here. */
+static void
+load_cups_hold_types (GPANode *parent)
+{
+	GPANode *node;
+	
+	node = gpa_option_list_new (parent, "Hold", "no-hold");
+
+	if (!node)
+		return;
+	
+	gpa_option_item_new (node,"weekend",_("on the weekend"));
+	gpa_option_item_new (node,"third-shift",
+			     _("between midnight and 8 a.m."));
+/* 	gpa_option_item_new (node,"night",_("between 6 p.m. and 6 a.m.")); */
+	gpa_option_item_new (node,"evening",_("between 6 p.m. and 6 a.m."));
+	gpa_option_item_new (node,"second-shift",
+			     _("between 4 p.m. and midnight"));
+	gpa_option_item_new (node,"day-time",_("between 6 a.m. and 6 p.m."));
+	gpa_option_item_new (node,"indefinite",_("when manually released"));
+	gpa_option_item_new (node,"no-hold",_("immediately"));
 }
 
 static GPAModel *
@@ -228,6 +308,7 @@ get_model (const gchar *printer, ppd_file_t *ppd)
 	GPANode *media;
 	GPANode *model;
 	GPANode *output;
+	GPANode *job;
 	char *xml;
 	char *id;
 
@@ -246,72 +327,51 @@ get_model (const gchar *printer, ppd_file_t *ppd)
 
 	output = gpa_node_lookup (model, "Options.Output");
 	media  = gpa_node_lookup (model, "Options.Output.Media");
+	job  = gpa_node_lookup (model, "Options.Output.Job");
 
 	load_paper_sizes   (ppd, media);
 	load_paper_sources (ppd, output);
+	load_cups_hold_types (job);
 
 	gpa_node_unref (output);
 	gpa_node_unref (media);
+	gpa_node_unref (job);
 
 	g_free (id);
 	
 	return (GPAModel *)model;
 }
 
+
 static gboolean
-append_printer (GPAList *printers_list, const char *name, gboolean is_default)
+append_printer (GPAList *printers_list, const char *name, 
+		gboolean is_default, const gchar *path)
 {
-	GPANode *settings = NULL;
 	GPANode *printer  = NULL;
-	GPAModel *model    = NULL;
-	const char *filename;
-	ppd_file_t *ppd;
 	gboolean retval = FALSE;
 
-	filename = cupsGetPPD (name);
-	ppd = ppdOpenFile (filename);
-	if (ppd == NULL)
-		/* See bug#: 102938 */
-		return FALSE;
+	printer = gpa_printer_new_stub (name, name, path);
 
-	model = get_model (name, ppd);
-	if (model == NULL)
-		goto append_printer_exit;
-
-	settings = gpa_settings_new (model, "Default", "SettIdFromCups");
-	if (settings == NULL)
-		goto append_printer_exit;
-
-	printer = gpa_printer_new (name, name, model, GPA_SETTINGS (settings));
-	if (printer == NULL)
-		goto append_printer_exit;
-
-	if (gpa_node_verify (printer)) {
+	if (printer != NULL && gpa_node_verify (printer)) {
 		gpa_list_prepend (printers_list, printer);
-		if (is_default)
+		if (is_default) {
 			gpa_list_set_default (printers_list, printer);
+			gpa_module_load_data (GPA_PRINTER (printer));
+		}
 		retval = TRUE;
 	}
 
- append_printer_exit:
 	if (retval == FALSE) {
 		g_warning ("The CUPS printer %s could not be created\n", name);
-
 		my_gpa_node_unref (printer);
-		my_gpa_node_unref (GPA_NODE (model));
-		my_gpa_node_unref (settings);
 	}
 
-	if (ppd) {
-		ppdClose (ppd);
-		unlink (filename);
-	}
-	
 	return retval;
 }
 
 static void
-gnome_print_cups_printer_list_append (gpointer printers_list)
+gnome_print_cups_printer_list_append (gpointer printers_list, 
+				      const gchar *path)
 {
 	/* const gchar *def = cupsGetDefault(); 
 	   not reliable, dests[i].is_default used instead */
@@ -326,7 +386,8 @@ gnome_print_cups_printer_list_append (gpointer printers_list)
 	
 	for (i = 0; i < num_printers; i ++)
 		append_printer (GPA_LIST (printers_list),
-				dests[i].name, dests[i].is_default);
+				dests[i].name, dests[i].is_default,
+				path);
 	
 	cupsFreeDests (num_printers, dests);
 	
@@ -334,6 +395,60 @@ gnome_print_cups_printer_list_append (gpointer printers_list)
 }
 
 
+/*  ------------- GPA load_data ------------- */
+
+G_MODULE_EXPORT void gpa_module_load_data (GPAPrinter *printer)
+{
+	GPANode *settings = NULL;
+	GPAModel *model    = NULL;
+	char *filename     = NULL;
+	const char *name   = printer->name;
+	ppd_file_t *ppd    = NULL;
+	gboolean success   = FALSE;
+
+	if (printer->is_complete)
+		return;
+
+	filename = g_strdup (cupsGetPPD (name)); 
+	ppd = ppdOpenFile (filename);
+
+	if (ppd == NULL)
+		goto gpa_module_load_data_exit;
+
+	model = get_model (name, ppd);
+	if (model == NULL)
+		goto gpa_module_load_data_exit;
+
+	settings = gpa_settings_new (model, "Default", "SetIdFromCups");
+	if (settings == NULL)
+		goto gpa_module_load_data_exit;
+
+	success = gpa_printer_complete_stub (printer, model, 
+					     GPA_SETTINGS (settings));
+
+ gpa_module_load_data_exit:
+	if (success == FALSE) {
+		g_warning ("The data for the CUPS printer %s "
+			   "could not be loaded.", name);
+
+		if (model != NULL)
+			my_gpa_node_unref (GPA_NODE (model));
+		if (settings != NULL)
+			my_gpa_node_unref (settings);
+	}
+
+	if (ppd) {
+		ppdClose (ppd);
+		unlink (filename);
+	}
+
+	if (filename)
+		g_free (filename);
+	
+	return;
+}
+
+
 /*  ------------- GPA init ------------- */
 G_MODULE_EXPORT gboolean gpa_module_init (GpaModuleInfo *info);
 
